@@ -22,14 +22,13 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from itertools import chain
 import re
 
 from gi.repository import Gio, GLib
 
-from gnomemusic.albumartcache import lookup_art_file_from_cache
+from gnomemusic.grilowrappers.grltrackerplaylists import Playlist
 from gnomemusic.gstplayer import Playback
-from gnomemusic.player import PlayerPlaylist, RepeatMode
+from gnomemusic.player import RepeatMode
 from gnomemusic.widgets.songwidget import SongWidget
 
 
@@ -305,6 +304,10 @@ class MPRIS(DBusInterface):
         self._playlists_model.connect(
             "items-changed", self._on_playlists_items_changed)
 
+        self._recent_playlist = self._coremodel.props.recent_playlist
+        self._recent_playlist.connect(
+            "items-changed", self._on_recent_playlist_changed)
+
         self._player_playlist_type = None
         self._path_list = []
         self._metadata_list = []
@@ -372,21 +375,8 @@ class MPRIS(DBusInterface):
         if track_nr > 0:
             metadata['xesam:trackNumber'] = GLib.Variant('i', track_nr)
 
-        # If the media has already been part of an MPRIS playlist, its
-        # thumbnail is already set. Otherwise, try to look for it in the
-        # cache directory and set the media thumbnail for a future use.
-        # The search is only through the cache to prevent any delayed
-        # loading.
-        # FIXME: The thumbnail retrieval should take place in the
-        # player.
-        art_url = coresong.props.media.get_thumbnail()
-        if not art_url:
-            thumb_file = lookup_art_file_from_cache(coresong)
-            if thumb_file:
-                art_url = GLib.filename_to_uri(thumb_file.get_path())
-                coresong.props.media.set_thumbnail(art_url)
-
-        if art_url:
+        art_url = coresong.props.thumbnail
+        if art_url not in ["generic", "loading"]:
             metadata['mpris:artUrl'] = GLib.Variant('s', art_url)
 
         return metadata
@@ -417,51 +407,20 @@ class MPRIS(DBusInterface):
             id_hex, index)
         return path
 
-    def _update_tracklist(self):
-        previous_path_list = self._path_list
+    def _on_recent_playlist_changed(self, model, position, removed, added):
         self._path_list = []
         self._metadata_list = []
-        current_position = self._player.props.position
-        nb_songs = self._player_model.get_n_items()
 
-        index_min = current_position - self._playlist_nb_songs
-        index_max = current_position + self._playlist_nb_songs + 1
-        if self._player_playlist_type == PlayerPlaylist.Type.ALBUM:
-            index_min = 0
-            index_max = self._player_model.get_n_items()
+        offset = self._recent_playlist.get_offset()
+        for position, coresong in enumerate(self._recent_playlist):
+            offset_position = position + offset
+            self._path_list.append(
+                self._get_song_dbus_path(coresong, offset_position))
+            self._metadata_list.append(
+                self._get_metadata(coresong, offset_position))
 
-        first_index = max(index_min, 0)
-        last_index = min(index_max, nb_songs)
-        positions = range(first_index, last_index)
-
-        nb_songs_max = 2 * self._playlist_nb_songs + 1
-        if (self._player.props.repeat_mode == RepeatMode.ALL
-                and (last_index - first_index) < nb_songs_max):
-            offset_sup = min(
-                self._playlist_nb_songs - last_index + current_position + 1,
-                first_index)
-            offset_inf = min(
-                self._playlist_nb_songs - current_position + first_index,
-                nb_songs - last_index)
-
-            positions = chain(
-                range(nb_songs - offset_inf, nb_songs), positions,
-                range(offset_sup))
-
-        for position in positions:
-            coresong = self._player_model.get_item(position)
-            path = self._get_song_dbus_path(coresong, position)
-            metadata = self._get_metadata(coresong, position)
-            self._path_list.append(path)
-            self._metadata_list.append(metadata)
-
-        # current song has changed
-        if (not previous_path_list
-                or previous_path_list[0] != self._path_list[0]
-                or previous_path_list[-1] != self._path_list[-1]
-                or len(previous_path_list) != len(self._path_list)):
-            current_song_path = self._get_song_dbus_path()
-            self._track_list_replaced(self._path_list, current_song_path)
+        current_song_path = self._get_song_dbus_path()
+        self._track_list_replaced(self._path_list, current_song_path)
 
     def _get_playlist_dbus_path(self, playlist):
         """Convert a playlist to a D-Bus path
@@ -475,6 +434,7 @@ class MPRIS(DBusInterface):
 
         # Smart Playlists do not have an id
         pl_id = playlist.props.pl_id or playlist.props.tag_text
+        pl_id = pl_id.rsplit(":")[-1].replace("-", "")
         return "/org/gnome/GnomeMusic/Playlist/{}".format(pl_id)
 
     def _get_mpris_playlist_from_playlist(self, playlist):
@@ -494,12 +454,12 @@ class MPRIS(DBusInterface):
         :returns: playlist existence and its structure
         :rtype: tuple
         """
-        current_playlist = self._coremodel.props.active_playlist
-        if current_playlist is None:
+        current_core_object = self._coremodel.props.active_core_object
+        if not isinstance(current_core_object, Playlist):
             return (False, ("/", "", ""))
 
         mpris_playlist = self._get_mpris_playlist_from_playlist(
-            current_playlist)
+            current_core_object)
         return (True, mpris_playlist)
 
     def _on_current_song_changed(self, player):
@@ -522,8 +482,6 @@ class MPRIS(DBusInterface):
         if added == model.get_n_items():
             return
 
-        self._update_tracklist()
-
         properties = {}
         properties["Metadata"] = GLib.Variant("a{sv}", self._get_metadata())
 
@@ -556,8 +514,6 @@ class MPRIS(DBusInterface):
             {'PlaybackStatus': GLib.Variant('s', playback_status), }, [])
 
     def _on_repeat_mode_changed(self, player, param):
-        self._update_tracklist()
-
         properties = {}
 
         is_shuffled = self._player.props.repeat_mode == RepeatMode.SHUFFLE
@@ -728,19 +684,27 @@ class MPRIS(DBusInterface):
         :param str path: Identifier of the track to skip to
         """
         current_index = self._path_list.index(self._get_song_dbus_path())
-        current_coresong = self._player_model[current_index]
+        current_coresong = self._player.props.current_song
 
         goto_index = self._path_list.index(path)
-        new_coresong = self._player_model[goto_index]
+        new_position = self._player.props.position + goto_index - current_index
+        new_coresong = self._player_model[new_position]
 
         self._player.play(new_coresong)
         current_coresong.props.state = SongWidget.State.PLAYED
         new_coresong.props.state = SongWidget.State.PLAYING
 
-    def _track_list_replaced(self, tracks, current_song):
+    def _track_list_replaced(self, track_paths, current_song_path):
+        """Indicate that the entire tracklist has been replaced.
+
+        (MPRIS Method)
+
+        :param list track_paths: the new list of tracks
+        :param current_song_path: the id of the current song
+        """
         parameters = {
-            "Tracks": self._path_list,
-            "CurrentTrack": self._get_song_dbus_path()}
+            "Tracks": track_paths,
+            "CurrentTrack": current_song_path}
         self._dbus_emit_signal("TrackListReplaced", parameters)
 
     def _load_player_playlist(self, playlist):
@@ -750,9 +714,7 @@ class MPRIS(DBusInterface):
 
         loaded_id = self._coremodel.connect(
             "playlist-loaded", _on_playlist_loaded)
-        self._coremodel.props.active_playlist = playlist
-        self._coremodel.set_player_model(
-            PlayerPlaylist.Type.PLAYLIST, playlist.props.model)
+        self._coremodel.props.active_core_object = playlist
 
     def _activate_playlist(self, playlist_path):
         """Starts playing the given playlist (MPRIS Method).
